const {XMLHttpRequest} = require('sdk/net/xhr');
const {setPref, getPref} = require('prefs');
const format = require('format');

let analytics = function() {
  let log = require('logging').create("analytics.js", "Debug");

  // Default value for the GA property
  let tid = "UA-59324753-1";

  /**
   * @return current client ID
   */
  function getTrackingClientId() {
    let trackingClientId = fetchClientId();
    if (trackingClientId == null) {
      trackingClientId = generateTrackingClientId();
      log.debug("generated new tracking client ID: " + trackingClientId);
      storeClientId(trackingClientId);
    }
    return trackingClientId;
  }

  /**
   * fetches client id from preferences store
   */
  function fetchClientId() {
    return getPref('tracking_cid', null);
  }

  /**
   * stores the client ID into preferences store
   */
  function storeClientId(value) {
    return setPref('tracking_cid', value);
  }

  /**
   * generates fresh new client ID
   *
   * @throws NS_ERROR_FAILURE
   */
  function generateTrackingClientId() {
    const uuid = require('utils/uuid_generator');
    return uuid.generate();
  }

  return {
    init : function(trackingId) {
      tid = trackingId;
    },

    trackEvent : function(action) {
      try {
        let xhr = new XMLHttpRequest();
        xhr.open("POST", "https://www.google-analytics.com/collect", true);
        xhr.onreadystatechange = function(e) {
          if (xhr.readyState == 4) {
            if (xhr.status >= 200 && xhr.status <= 299) {
            } else {
              log.error("GA: failed to track: " + xhr.status + " event: " + e);
            }
          }
        }
        xhr.setRequestHeader("Content-Type", "application/x-www-form-urlencoded");
        let cid = getTrackingClientId();
        let data = format("v=1&tid={0}&cid={1}&t=event&ec=extension&ea={2}&el=&ev=0", tid, cid, action);
        xhr.send(data);
      } catch (err) {
        log.error("GA: failed to send metrics: " + err);
      }
    }
  }
}();

module.exports = analytics;
