const  {Cc, Ci} = require('chrome');
const {when: unload} = require('sdk/system/unload');
const {encode:btoa} = require('sdk/base64');

var authenticator = function() {
  let log = require('logging').create('authenticator.js');
  var authHeader = null;

  return {

    observe : function(subject, topic, data) {
      try {
        var httpChannel = subject.QueryInterface(Ci.nsIHttpChannel);

        if (httpChannel instanceof Ci.nsIProxiedChannel) {
          var proxiedChannel = httpChannel.QueryInterface(Ci.nsIProxiedChannel);
          if (proxiedChannel && proxiedChannel.proxyInfo) {
            var proxyInfo = proxiedChannel.proxyInfo;
            if (proxyInfo.host === "127.0.0.1" && proxyInfo.port === 49736 && authHeader && authHeader.name && authHeader.value) {
              httpChannel.setRequestHeader(authHeader.name, authHeader.value, false);
            }
          }
        }
      } catch(e) {
        log.error(e);
      }
    },

    register : function(username, password) {
      try {
        authHeader = {
          name:  "Proxy-Authorization",
          value: "Basic " + btoa(username + ":" + password)
        };

        var observerService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
        observerService.addObserver(this, "http-on-modify-request", false);
        unload(this.unregister.bind(this));
      } catch (e) {
        log.error(e);
      }
    },

    unregister : function() {
      try {
        log.info("authenticator.unregister");
        var observerService = Cc["@mozilla.org/observer-service;1"].getService(Ci.nsIObserverService);
        observerService.removeObserver(this, "http-on-modify-request");
      } catch (e) {
        log.error(e);
      }
    }
  };
}();

module.exports = authenticator;
