let logging = require("logging");

let log = logging.create("main.js", "Debug");

const {getPref, setPref, setDefaultPrefs} = require("prefs");

const browsec = require('browsec');
const browsecApi = require('browsec_api');
const passwordManager = require('password_manager');
const authenticator = require('authenticator');
const ui = require('ui');
const retry = require('utils/retry');

const self = require('sdk/self');
if (self.loadReason == 'install') {
  browsec.install_addon();
}

browsec.init();
passwordManager.init(browsec.getLocalProxyHost(), browsec.getLocalProxyPort());

try {
  browsec.start();

  log.debug("Set default prefs");
  setDefaultPrefs();

  log.debug("Check credentials");
  browsec.status = 'logging in...';
  retry(60000, ensureValidCredentials, function(err, credentials) {
    if (err !== null) {
      log.error("Failed to sign up: " + err);
    } else {
      log.debug("Got valid credentials, proceeding");

      if (credentials) {
        authenticator.register(credentials.username, credentials.password);
      }
      browsec.status = 'logged in';
      browsec.enabled = true;

      browsec.enable_addon();
    }
  });
  ui.setup(browsec);
} catch (e) {
    log.error("Error in bootstrap startup: " + e);
}

function ensureValidCredentials(callback) {
  var credentials = passwordManager.getSavedCredentials();

  checkCredentials(credentials, function(err, credentialsAreValid) {
    if (err !== null) { return callback(err); }

    if (credentialsAreValid) {
      callback(null, credentials);
    } else {
      signup(callback);
    }
  });
}

function checkCredentials(credentials, callback) {
  if (credentials) {
    browsecApi.checkCredentials(credentials.username, credentials.password, callback)
  } else {
    callback(null, false);
  }
}

function signup(callback) {
  browsecApi.signup(function(err, result) {
    if (err !== null) { return callback(err); }

    let credentials = null;
    if (result) {
      credentials = {
        username: result.user_id,
        password: result.password
      };

      log.debug("Save credentials.");
      passwordManager.saveCredentials(credentials.username, credentials.password);
    } else {
      log.error("Can't signup user");
      callback("browsec_api.signup failed");
    }
    callback(null, credentials);
  });
}
