const {Cu} = require('chrome');
Cu.import('resource://gre/modules/Services.jsm');

let proxyService = function() {
  const utils = require('utils');
  let log = require('logging').create("proxy.js", "Debug");

  let backupProxyPrefs = [
    "type", 
    "http", 
    "http_port",
    "ssl",
    "ssl_port",
    "ftp",
    "ftp_port",
    "no_proxies_on",
    "share_proxy_settings",
    "autoconfig_url"
  ];

  return {
    backupBrowserProxySettings : function() {
      try {
        log.info("Backup FF proxy settings");

        let proxyPrefs = Services.prefs.getBranch("network.proxy.");
        let extensionBackupProxyPrefs = Services.prefs.getBranch("extensions.browsec.backup.network.proxy.");

        utils.copyPrefs(backupProxyPrefs, proxyPrefs, extensionBackupProxyPrefs);
      }
      catch(e) {
        log.error("Backup error: " + e);
      }
    },

    restoreBrowserProxySettings : function() {
      try {
        log.info("Restore FF proxy settings");
      
        let proxyPrefs = Services.prefs.getBranch("network.proxy.");
        let extensionBackupProxyPrefs = Services.prefs.getBranch("extensions.browsec.backup.network.proxy.");

        utils.copyPrefs(backupProxyPrefs, extensionBackupProxyPrefs, proxyPrefs);
      }
      catch(e) {
        log.error("Restore error: " + e);
      }
    },

    clearProxySettingsBackup : function() {
      try {
        log.info("Clear FF proxy settings backup");
      
        Services.prefs.deleteBranch("extensions.browsec.backup.network.proxy.");
      }
      catch(e) {
        log.error("Clear proxy settings backup error: " + e);
      }
    },

    isProxySet : function(proxyUrl) {
      try {
        log.info("isProxySet");
      
        let proxy = this.parseProxyUrl(proxyUrl);
        let proxyHost = proxy.host;
        let proxyPort = proxy.port;

        let proxyPrefs = Services.prefs.getBranch("network.proxy.");

        return proxyPrefs.getIntPref("type") === 1 && proxyPrefs.getCharPref("http") === proxyHost && proxyPrefs.getIntPref("http_port") === proxyPort;
      }
      catch(e) {
        log.error("isProxySet error: " + e);
      }
    },

    setProxy : function (proxyUrl, bypass) {
      bypass = typeof bypass !== 'undefined' ? bypass : "localhost, localdomain, .localdomain, local, .local, 127.0.0.1, 10.0.0.0/8, 172.16.0.0/12, 192.168.0.0/16";
      if (bypass.indexOf("www.google-analytics.com") < 0) {
        bypass += ", www.google-analytics.com";
      }

      try {

        let proxyPrefs = Services.prefs.getBranch("network.proxy.");

        if (proxyUrl == "DIRECT") {
          log.info("Set DIRECT proxy connection");
          proxyPrefs.setIntPref("type", 0);
        } else {
          log.info("Set proxy " + proxyUrl)
          proxyPrefs.setIntPref("type", 1);
          let proxy = this.parseProxyUrl(proxyUrl);
          let proxyHost = proxy.host;
          let proxyPort = proxy.port;
          proxyPrefs.setCharPref("http", proxyHost);
          proxyPrefs.setIntPref("http_port", proxyPort);
          proxyPrefs.setCharPref("ssl", proxyHost);
          proxyPrefs.setIntPref("ssl_port", proxyPort);
          proxyPrefs.setCharPref("ftp", proxyHost);
          proxyPrefs.setIntPref("ftp_port", proxyPort);
          proxyPrefs.setBoolPref("share_proxy_settings", false);
          proxyPrefs.setCharPref("no_proxies_on", bypass);
        }
      } catch (e) {
        log.error("Can't set proxy " + e);
      }
    },

    parseProxyUrl : function(proxyUrl) {
      let proxyParts = proxyUrl.split(':');
      let proxyHost = proxyParts[0];
      let proxyPort = proxyParts[1];

      return {host : proxyHost, port : proxyPort};
    },

    getBypass : function() {
      try {
        let proxyPrefs = Services.prefs.getBranch("network.proxy.");
        return proxyPrefs.getCharPref("no_proxies_on");
      } catch (e) {
        log.error("Can't get bypass: " + e);
        return null;
      }
    }
  };
}();

module.exports = proxyService;
