const { getPref, setPref } = require('prefs');
const {ActionButton} = require('sdk/ui/button/action');
const log = require('logging').create('ui.js', 'Debug');

function getToolbarIconPath(active) {
  return active ? "./images/toolbar-icon-active.png" : "./images/toolbar-icon-deactive.png";
}

function statusView(status) {
    if (typeof status === 'string') {
      return "Browsec: " + status;
    } else {
      return "Browsec";
    }
}

function setup(browsec) {
  let button = ActionButton({
    id: "browsec-tbb",
    label: statusView(browsec.status),
    icon: getToolbarIconPath(browsec.active),
    disabled: !browsec.enabled,
    onClick: function clickToolbarButton() {
      browsec.toggleActiveState();
    }
  });

  browsec.on('enabled', function(enabled) {
    button.disabled = !enabled;
  });

  browsec.on('status', function(status) {
    button.label = statusView(status);
  });

  browsec.on('active', function(newActiveState) {
    let iconPath = getToolbarIconPath(newActiveState);
    button.icon = iconPath;
  });

  let firstrun = getPref("firstrun");
  if (firstrun) {
    setPref("firstrun", false);
    log.debug("Open firstrun page");
    require('sdk/tabs').open("https://browsec.com/congratulations?addon");
  }
}

exports.setup = setup;
