const {setTimeout} = require('utils/timer');
const log = require('logging').create('retry.js', 'Debug');

/**
 * Retries given `task` indefinitely with `timeout` ms between attempts.
 *
 * @param timeout
 *    Timeout between attempts, in milliseconds
 * @param task
 *    a function accepting a NodeJS-style callback
 * callback
 *    a nodejs-style callback to be called when results are ready.
 */
module.exports = function retry(timeout, task, callback) {
  (function attempt() {
    task(function(err, ...args) {
      if (err !== null) {
        log.debug('Failed attempt, retrying in ' + timeout + 'ms: ' + task.name);
        setTimeout(attempt, timeout);
      } else {
        callback(null, ...args);
      }
    });
  })();
}
